<?php
/**
 * A general index class
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2012 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
class BaseIndex extends BaseFrameworkSystem {
	/**
	 * Magic for this index
	 */
	const INDEX_MAGIC = 'INDEXv0.1';

	/**
	 * Separator for header data
	 */
	const SEPARATOR_HEADER_DATA = 0x01;

	/**
	 * Separator header->entries
	 */
	const SEPARATOR_HEADER_ENTRIES = 0x02;

	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of the class
	 * @return	void
	 */
	protected function __construct ($className) {
		// Call parent constructor
		parent::__construct($className);

		// Calculate header size
		$this->setHeaderSize(
			strlen(self::INDEX_MAGIC) +
			strlen(self::SEPARATOR_HEADER_DATA) +
			self::LENGTH_COUNT +
			strlen(self::SEPARATOR_HEADER_ENTRIES)
		);

		// Init counters and gaps array
		$this->initCountersGapsArray();
	}

	/**
	 * Reads the file header
	 *
	 * @return	void
	 */
	protected function readFileHeader () {
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] CALLED!', __METHOD__, __LINE__));

		// First rewind to beginning as the header sits at the beginning ...
		$this->getIteratorInstance()->rewind();

		// Then read it (see constructor for calculation)
		$data = $this->getIteratorInstance()->read($this->getHeaderSize());
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] Read %d bytes (%d wanted).', __METHOD__, __LINE__, strlen($data), $this->getHeaderSize()));

		// Have all requested bytes been read?
		assert(strlen($data) == $this->getHeaderSize());
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] Passed assert().', __METHOD__, __LINE__));

		// Last character must be the separator
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] data(-1)=%s', __METHOD__, __LINE__, dechex(ord(substr($data, -1, 1)))));
		assert(substr($data, -1, 1) == chr(self::SEPARATOR_HEADER_ENTRIES));
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] Passed assert().', __METHOD__, __LINE__));

		// Okay, then remove it
		$data = substr($data, 0, -1);

		// And update seek position
		$this->updateSeekPosition();

		/*
		 * Now split it:
		 *
		 * 0 => magic
		 * 1 => total entries
		 */
		$header = explode(chr(self::SEPARATOR_HEADER_DATA), $data);

		// Set it here
		$this->setHeader($header);

		// Check if the array has only 3 elements
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] header(%d)=%s', __METHOD__, __LINE__, count($header), print_r($header, TRUE)));
		assert(count($header) == 2);
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] Passed assert().', __METHOD__, __LINE__));

		// Check magic
		assert($header[0] == self::INDEX_MAGIC);
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] Passed assert().', __METHOD__, __LINE__));

		// Check length of count
		assert(strlen($header[1]) == self::LENGTH_COUNT);
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] Passed assert().', __METHOD__, __LINE__));

		// Decode count
		$header[1] = hex2bin($header[1]);

		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] EXIT!', __METHOD__, __LINE__));
	}

	/**
	 * Flushes the file header
	 *
	 * @return	void
	 */
	private function flushFileHeader () {
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] CALLED!', __METHOD__, __LINE__));

		// Put all informations together
		$header = sprintf('%s%s',
			// Magic
			self::INDEX_MAGIC,

			// Separator position<->entries
			chr(self::SEPARATOR_HEADER_ENTRIES)
		);

		// Write it to disk (header is always at seek position 0)
		$this->writeData(0, $header);

		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] EXIT!', __METHOD__, __LINE__));
	}

	/**
	 * Analyzes entries in index file. This will count all found (and valid)
	 * entries, mark invalid as damaged and count gaps ("fragmentation"). If
	 * only gaps are found, the file is considered as "virgin" (no entries).
	 *
	 * @return	void
	 */
	private function analyzeFile () {
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] CALLED!', __METHOD__, __LINE__));

		// Make sure the file is initialized
		assert($this->isFileInitialized());

		// Init counters and gaps array
		$this->initGapsArray();

		// Output message (as this may take some time)
		self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] Analyzing file structure ... (this may take some time)', __METHOD__, __LINE__));

		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(sprintf('[%s:%d:] EXIT!', __METHOD__, __LINE__));
	}

	/**
	 * Initializes this index
	 *
	 * @param	$fileName	File name of this index
	 * @return	void
	 */
	protected function initIndex ($fileName) {
		// Get a file i/o pointer instance for index file
		$pointerInstance = ObjectFactory::createObjectByConfiguredName('file_raw_input_output_class', array($fileName));

		// Get iterator instance
		$iteratorInstance = ObjectFactory::createObjectByConfiguredName('file_io_iterator_class', array($pointerInstance));

		// Is the instance implementing the right interface?
		assert($iteratorInstance instanceof SeekableWritableFileIterator);

		// Set iterator here
		$this->setIteratorInstance($iteratorInstance);

		// Is the file's header initialized?
		if (!$this->isFileHeaderInitialized()) {
			// No, then create it (which may pre-allocate the index)
			$this->createFileHeader();

			// And pre-allocate a bit
			$this->preAllocateFile('index');
		} // END - if

		// Load the file header
		$this->readFileHeader();

		// Count all entries in file
		$this->analyzeFile();
	}
}

// [EOF]
?>
